/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * Project Name : HY16F3910_Power
 * IDE tooling  : AndeSight C/C++ IDE, version: 3.2.1
 * Device Ver.  :
 * Library Ver. : 0.5
 * MCU Device   :
 * Description  :
 * Created Date : 2022/03/22
 * Created by   :
 *
 * Program Description:
 *
 *   HY16F3910
 *----------------          ----------------
 *                |        |
 *          PT1.0 | <----- | button0 (Wake up)
 *          PT1.1 | <----- | button1 (IDLE or SLEEP)
 *                |        |
 *                |          ----------------
 *                |
 *                |           LCD Board(HY10000-AM01)
 *                |         ----------------
 *                |        |
 *  PT13.4~PT13.5 | -----> | SEG0~SEG1
 *    PT6.0~PT6.7 | -----> | SEG2~SEG9
 *    PT7.0~PT7.7 | -----> | SEG10~SEG17
 *  PT13.0~PT13.3 | -----> | COM0~COM3
 *                |        |
 *                |         ----------------
 *----------------
 *
 * NOTE: Sleep/Idle Current is measured in free run
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "HY391apb.h"
#include "DrvCLOCK.h"
#include "DrvLCD.h"
#include "Display.h"
#include "main.h"
#include "System.h"
#include "DrvGPIO.h"
#include "DrvPMU.h"
/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/
typedef union _PTINTSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_PTINT0done:1;
    unsigned b_PTINT1done:1;
    unsigned b_PTINT2done:1;
    unsigned b_PTINT3done:1;
    unsigned b_PTINT4done:1;
    unsigned b_PTINT5done:1;
    unsigned b_PTINT6Done:1;
    unsigned b_PTINT7Done:1;
  };
} PTINTSTATUS;

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#define KEY_PORT E_PT1
#define KEYIN0 BIT0
#define KEYIN1 BIT1

//#define HAO_4MHZ
#define HAO_32MHZ

#define IDLE_MODE
//#define SLEEP_MODE
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
PTINTSTATUS  PT1INTSTATUSbits;

/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);

/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
int main(void)
{
  DrvPMU_VDD15Trim();  // Make sure VDD15 Voltage,before change OSC setting
  DrvCLOCK_EnableHighOSC(E_INTERNAL,100);     // Select HSRC
#if defined(HAO_4MHZ)
  DrvCLOCK_SelectIHOSC_CalHAO(E_HAO_4M);                 //Select internal 4.147MHz, and calibration 4.147MHz
  DrvCLOCK_SelectMCUClock(E_HSCK,MCUCKDIV1);                //ENMCD=01b,MCU Clock/1, CPU CLOCK IS 'HS_CK/1'
#elif defined(HAO_32MHZ)
  DrvCLOCK_SelectIHOSC_CalHAO(E_HAO_32M);                 //Select internal 31.795MHz, and calibration 31.795MHz
  DrvCLOCK_SelectMCUClock(E_HSCK,MCUCKDIV2);                //ENMCD=00b,MCU Clock/2, CPU CLOCK IS 'HS_CK/2'
#endif
  DisplayInit();
  ClearLCDframe();
  LCD_DATA_DISPLAY(888888);
  Delay(50000);
  Delay(50000);
  DrvGPIO_Open(KEY_PORT,KEYIN1|KEYIN0,E_IO_INPUT);      //set PT1.0/PT1.1 INPUT
  DrvGPIO_Open(KEY_PORT,KEYIN1|KEYIN0,E_IO_PullHigh);   //enable PT1.0/PT1.1 pull high R
  DrvGPIO_Open(KEY_PORT,KEYIN0,E_IO_IntEnable);  //PT1.0 interrupt enable
  DrvGPIO_IntTrigger(KEY_PORT,KEYIN0,E_N_Edge);  //PT1.0 interrupt trigger method is negative edge
  DrvGPIO_ClearIntFlag(KEY_PORT,KEYIN0);         //clear PT1 interrupt flag
  PT1INTSTATUSbits._byte = 0;
  SYS_EnableGIE(4,0x3BF);                               //Enable GIE(Global Interrupt)

  while(1)
  {
    DisplayHYcon();
    Delay(10000);
    if((DrvGPIO_PT1_GetPortBits()&KEYIN1)==0)           //if PT1.1 low, Enter Sleep or Idle Mode
    {
      //Close LCD
      DrvLCD_LCDEnable(0);    //LCDEN=0b
      DrvLCD_VLCDEnable(0);   //VLCDEN=0b
      DrvLCD_DisplayMode(2);  //DSP=10b
      DrvLCD_LCDBuffer(0);    //BEn=0b
      while((inw(0x41B00)&(1<<IDF))==0);                //Wait LCD Idle, IDF=20
      //Power
      DrvPMU_DisableBOR2();
      //Enter IDLE or SLEEP
      #if defined(IDLE_MODE)  //Idle Mode
      SYS_LowPower(SYS_IdleMode,0);  //0: HAO Disable 1:HAO Enable
      #endif
      #if defined(SLEEP_MODE)  //Sleep Mode
      DrvPMU_LDO_LowPower(1);   //Low Power mode,only support SLEEP mode
      SYS_LowPower(SYS_SleepMode,0); //0: LPO Disable 1:LPO Enable
      DrvPMU_LDO_LowPower(0);   //SET normal power mode
      #endif
      //If wake up from Sleep or Idle mode, user can enable internal HAO first to do some application
      DrvPMU_EnableBOR2(1);    // Enable BOR2, Suggest to entering "1" for delay about 300us. The delay time is 150us per increases
      DisplayInit();
    }
  }
}

/*----------------------------------------------------------------------------*/
/* Function Name: HW0_ISR()                                                   */
/* Description  : I2C/UART/SPI interrupt Service Routine (HW0).               */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW0_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW1_ISR()                                                   */
/* Description  : WDT & RTC & Timer A/B/C interrupt Service Routine (HW1).    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW1_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW2_ISR()                                                   */
/* Description  : ADC interrupt Service Routine (HW2).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW3_ISR()                                                   */
/* Description  : LVD & BOR2 interrupt Service Routine (HW3).                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW3_ISR(void)
{

}

/*----------------------------------------------------------------------------*/
/* Function Name: HW4_ISR()                                                   */
/* Description  : PT1 interrupt Service Routine (HW4).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW4_ISR(void)
{
	uint32_t PORT_IntFlag;

	PORT_IntFlag=DrvGPIO_GetIntFlag(E_PT1);
	if((PORT_IntFlag&KEYIN0)==KEYIN0)
	{
	  PT1INTSTATUSbits.b_PTINT0done=1;
	}
	DrvGPIO_ClearIntFlag(KEY_PORT,KEYIN0);         //clear PT1.0 interrupt flag
}

/*----------------------------------------------------------------------------*/
/* Function Name: HW5_ISR()                                                   */
/* Description  : PT2 interrupt Service Routine (HW5).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW7_ISR()                                                   */
/* Description  : UART2 interrupt Service Routine (HW7).                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW7_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW8_ISR()                                                   */
/* Description  : TMB2 interrupt Service Routine (HW8).                       */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW8_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: HW9_ISR()                                                   */
/* Description  : PT3 interrupt Service Routine (HW9).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW9_ISR(void)
{

}
/*----------------------------------------------------------------------------*/
/* Function Name: general_exception_handler()                                 */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void general_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
  while(1);
}
/*----------------------------------------------------------------------------*/
/* Function Name: debug_exception_handler()                                   */
/* Description  : Exception Service Routines.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void debug_exception_handler()
{
  asm("nop"); //procedure define by customer.
  asm("nop");
  while(1);
}

/*----------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                 */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
  asm("NOP");
}

/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
